@@ -31,6 +31,14 @@ impl<T> PrivateSeries for SeriesWrap<ObjectChunked<T>>
 where
     T: PolarsObject,
 {
+    fn agg_first(&self, _groups: &[(u32, Vec<u32>)]) -> Series {
+        self.0.agg_first(_groups)
+    }
+
+    fn agg_last(&self, _groups: &[(u32, Vec<u32>)]) -> Series {
+        self.0.agg_last(_groups)
+    }
+
     fn str_value(&self, index: usize) -> Cow<str> {
         match (&self.0).get(index) {
             None => Cow::Borrowed("null"),
@@ -59,6 +67,11 @@ impl<T> SeriesTrait for SeriesWrap<ObjectChunked<T>>
 where
     T: PolarsObject,
 {
+    #[cfg(feature = "interpolate")]
+    fn interpolate(&self) -> Series {
+        self.0.interpolate().into_series()
+    }
+
     fn rename(&mut self, name: &str) {
         ObjectChunked::rename(&mut self.0, name)
     }
