@@ -24,6 +24,16 @@ impl PyLazyGroupBy {
         lgb.agg(aggs).into()
     }
 
+    pub fn head(&mut self, n: usize) -> PyLazyFrame {
+        let lgb = self.lgb.take().unwrap();
+        lgb.head(Some(n)).into()
+    }
+
+    pub fn tail(&mut self, n: usize) -> PyLazyFrame {
+        let lgb = self.lgb.take().unwrap();
+        lgb.tail(Some(n)).into()
+    }
+
     pub fn apply(&mut self, lambda: PyObject) -> PyLazyFrame {
         let lgb = self.lgb.take().unwrap();
 
@@ -37,7 +47,7 @@ impl PyLazyGroupBy {
             let pydf = PyDataFrame::new(df);
 
             // Wrap this PySeries object in the python side DataFrame wrapper
-            let python_df_wrapper = pypolars.call1("wrap_df", (pydf,)).unwrap();
+            let python_df_wrapper = pypolars.getattr("wrap_df").unwrap().call1((pydf,)).unwrap();
 
             // call the lambda and get a python side DataFrame wrapper
             let result_df_wrapper = match lambda.call1(py, (python_df_wrapper,)) {
@@ -204,10 +214,14 @@ impl PyLazyFrame {
         ldf.select(exprs).into()
     }
 
-    pub fn groupby(&mut self, by: Vec<PyExpr>) -> PyLazyGroupBy {
+    pub fn groupby(&mut self, by: Vec<PyExpr>, maintain_order: bool) -> PyLazyGroupBy {
         let ldf = self.ldf.clone();
         let by = py_exprs_to_exprs(by);
-        let lazy_gb = ldf.groupby(by);
+        let lazy_gb = if maintain_order {
+            ldf.stable_groupby(by)
+        } else {
+            ldf.groupby(by)
+        };
 
         PyLazyGroupBy { lgb: Some(lazy_gb) }
     }
@@ -281,6 +295,11 @@ impl PyLazyFrame {
         ldf.fill_none(fill_value.inner).into()
     }
 
+    pub fn fill_nan(&self, fill_value: PyExpr) -> Self {
+        let ldf = self.ldf.clone();
+        ldf.fill_nan(fill_value.inner).into()
+    }
+
     pub fn min(&self) -> Self {
         let ldf = self.ldf.clone();
         ldf.min().into()
@@ -321,10 +340,10 @@ impl PyLazyFrame {
         ldf.quantile(quantile).into()
     }
 
-    pub fn explode(&self, column: Vec<String>) -> Self {
+    pub fn explode(&self, column: Vec<PyExpr>) -> Self {
         let ldf = self.ldf.clone();
-        let column = column.into_iter().map(|s| col(&s)).collect::<Vec<_>>();
-        ldf.explode(&column).into()
+        let column = py_exprs_to_exprs(column);
+        ldf.explode(column).into()
     }
 
     pub fn drop_duplicates(&self, maintain_order: bool, subset: Option<Vec<String>>) -> Self {
@@ -366,9 +385,9 @@ impl PyLazyFrame {
             // get the pypolars module
             let pypolars = PyModule::import(py, "polars").unwrap();
             // create a PyDataFrame struct/object for Python
-            let pyseries = PyDataFrame::new(s);
+            let pydf = PyDataFrame::new(s);
             // Wrap this PyDataFrame object in the python side DataFrame wrapper
-            let python_df_wrapper = pypolars.call1("wrap_df", (pyseries,)).unwrap();
+            let python_df_wrapper = pypolars.getattr("wrap_df").unwrap().call1((pydf,)).unwrap();
             // call the lambda and get a python side Series wrapper
             let result_df_wrapper = match lambda.call1(py, (python_df_wrapper,)) {
                 Ok(pyobj) => pyobj,
@@ -402,4 +421,13 @@ impl PyLazyFrame {
     pub fn clone(&self) -> PyLazyFrame {
         self.ldf.clone().into()
     }
+
+    pub fn columns(&self) -> Vec<String> {
+        self.ldf
+            .schema()
+            .fields()
+            .iter()
+            .map(|fld| fld.name().to_string())
+            .collect()
+    }
 }
